<?php

namespace Drupal\better_unpublished_terms\Plugin\EntityReferenceSelection;

use Drupal\Core\Entity\Query\QueryInterface;
use Drupal\taxonomy\Plugin\EntityReferenceSelection\TermSelection;

/**
 * Provides a selection handler for taxonomy terms that allows unpublished terms
 * to be referenced if the user has the appropriate permissions.
 */
class BetterUnpublishedTermSelection extends TermSelection {

  /**
   * {@inheritdoc}
   */
  public function buildEntityQuery($match = NULL, $match_operator = 'CONTAINS', $limit = 0): QueryInterface {
    $query = parent::buildEntityQuery($match, $match_operator);
    // Add a condition to include unpublished terms if the user has the
    // permission to edit/create/delete terms in this vocabulary.
    $see_unpublished = FALSE;
    if ($this->currentUser->hasPermission('administer taxonomy')) {
      $see_unpublished = TRUE;
    }
    else {
      $bundles = $this->entityTypeBundleInfo->getBundleInfo('taxonomy_term');
      foreach ($bundles as $bundle => $bundle_info) {
        if ($this->currentUser->hasPermission("edit terms in $bundle") || $this->currentUser->hasPermission("delete terms in $bundle") || $this->currentUser->hasPermission("create terms in $bundle")) {
          $see_unpublished = TRUE;
          break;
        }
      }
    }

    if (!$see_unpublished) {
      $query->condition('status', 1);
    }

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function validateReferenceableNewEntities(array $entities): array {
    $entities = array_filter($entities, function ($entity) {
      $target_bundles = $this->getConfiguration()['target_bundles'];
      if (isset($target_bundles)) {
        return in_array($entity->bundle(), $target_bundles);
      }
      return TRUE;
    });
    // Mirror the conditions checked in buildEntityQuery().
    if (!$this->currentUser->hasPermission('administer taxonomy')) {
      $user = $this->currentUser;
      $entities = array_filter($entities, function($term) use ($user) {
        /** @var \Drupal\taxonomy\TermInterface $term */

        if ($user->hasPermission("edit terms in {$term->bundle()}") ||
          $user->hasPermission("delete terms in {$term->bundle()}") ||
          $user->hasPermission("create terms in {$term->bundle()}")) {
          return TRUE;
        }

        return $term->isPublished();
      });
    }
    return $entities;
  }

}
